from itertools import chain
from box.functools import cachedproperty
from box.glob import enhanced_iglob
from box.os import enhanced_join, enhanced_walk
from ..find import find
from .emitter import FileEmitter
from .filename import FilenameConstraint
from .filepath import FilepathConstraint
from .maxdepth import MaxdepthConstraint


class find_files(find):
    """Find files using map_reduce framework.

    Parameters
    ----------
    filters: list
        Find filters.
    join: bool
        If True joins resulted filepath with basedir.
    basedir: str
        Base directory to find.
    filepathes: list
        List of filepathes or globs where to find.
    params: dict
        Parameters of map_reduce.

    Returns
    -------
    mixed:
        Found files after map_reduce.
    """

    # Public

    default_emitter = FileEmitter

    def __init__(self, *, join=False, basedir=None, filepathes=None, **params):
        self.__join = join
        self.__basedir = basedir
        self.__filepathes = filepathes
        super().__init__(**params)

    # Protected

    @cachedproperty
    def _values(self):
        for filepath in self._filepathes:
            # Emits every file in filepathes
            file = filepath
            if self.__join:
                file = enhanced_join(self.__basedir, filepath)
            yield self._emitter(
                file, basedir=self.__basedir, filepath=filepath)

    @cachedproperty
    def _constraints(self):
        constraints = [
            MaxdepthConstraint(),
            FilenameConstraint(),
            FilepathConstraint(self.__basedir)]
        constraints += super()._constraints
        return constraints

    @cachedproperty
    def _filepathes(self):
        if self.__filepathes is not None:
            # We have pathes or globs
            # TODO: fix it's not LAZY load
            chunks = []
            for filepath in self.__filepathes:
                chunk = enhanced_iglob(
                    filepath,
                    basedir=self.__basedir, sorter=sorted, mode='files')
                chunks.append(chunk)
            filepathes = chain(*chunks)
        else:
            # We have to walk fully
            filepathes = enhanced_walk(
                basedir=self.__basedir, sorter=sorted, mode='files')
        return filepathes
