from builtins import open
from functools import partial
from box.functools import cachedproperty
from box.os import enhanced_join
from box.types import RegexCompiledPatternType
from ..find import find
from ..files import find_files
from .emitter import StringEmitter


class find_strings(find):
    """Find strings in files using map_reduce framework.

    Parameters
    ----------
    string: str/re
        Include string pattern.
    filters: list
        Find filters.
    basedir: str
        Base directory to find.
    filepathes: list
        List of filepathes or globs where to find.
    params: dict
        Params of map_reduce.

    Returns
    -------
    mixed
        Found files after map_reduce.
    """

    # Public

    default_emitter = StringEmitter

    def __init__(self, *, string=None, basedir=None, filepathes=None, **params):
        self.__string = string
        self.__basedir = basedir
        self.__filepathes = filepathes
        super().__init__(**params)

    # Protected

    @cachedproperty
    def _values(self):
        for filepath in self._filepathes:
            # Reads every file in filepathes
            full_filepath = enhanced_join(self.__basedir, filepath)
            partial_emitter = partial(
                self._emitter, basedir=self.__basedir, filepath=filepath)
            with open(full_filepath) as fileobj:
                filetext = fileobj.read()
                if isinstance(self.__string, RegexCompiledPatternType):
                    # Search string is regex object - re search
                    for match in self.__string.finditer(filetext):
                        matched_groups = match.groups()
                        if matched_groups:
                            # Emits every matched group
                            for matched_group in matched_groups:
                                yield partial_emitter(matched_group)
                        else:
                            # Emits whole match
                            matched_string = match.group()
                            yield partial_emitter(matched_string)
                elif self.__string:
                    # Search string is string - str search
                    matches = filetext.count(self.__string)
                    for _ in range(matches):
                        # Emits given strings matches count times
                        yield partial_emitter(self.__string)
                else:
                    # Search string is None - no search
                    # Emits whole file
                    yield partial_emitter(filetext)

    @cachedproperty
    def _filepathes(self):
        filepathes = find_files(
            basedir=self.__basedir,
            filepathes=self.__filepathes,
            filters=self._filters)
        return filepathes
