import re
import unittest
from functools import partial
from box.functools import cachedproperty
from unittest.mock import Mock, patch
from importlib import import_module
component = import_module('run_find.files.find')


class find_files_Test(unittest.TestCase):

    # Actions

    def setUp(self):
        self.addCleanup(patch.stopall)
        self.filepathes = [
            'file1', 'file2', 'dir/file1', 'dir/file2', 'dir/subdir/file3']
        patch.object(component, 'enhanced_iglob', self.mock_enhanced_iglob).start()
        patch.object(component, 'enhanced_walk', self.mock_enhanced_walk).start()
        self.pfind = partial(component.find_files, reducers=[list])

    # Helpers

    @cachedproperty
    def mock_enhanced_iglob(self):
        return Mock(return_value=self.filepathes)

    @cachedproperty
    def mock_enhanced_walk(self):
        return Mock(return_value=self.filepathes)

    # Tests

    def test(self):
        files = self.pfind()
        self.assertEqual(files, self.filepathes)

    def test_with_filename(self):
        files = self.pfind(
            filters=[{'filename': 'file3'}])
        self.assertEqual(files, ['dir/subdir/file3'])

    def test_with_filename_is_regex(self):
        files = self.pfind(
            filters=[{'filename': re.compile('file1+')}, {'maxdepth': 1}])
        self.assertEqual(files, ['file1'])

    def test_with_filepath(self):
        files = self.pfind(
            filters=[{'filepath': 'file*'}])
        self.assertEqual(files, ['file1', 'file2'])

    def test_with_filepath_is_regex(self):
        files = self.pfind(
            filters=[{'filepath': re.compile('.*2$')}])
        self.assertEqual(files, ['file2', 'dir/file2'])

    def test_with_basedir(self):
        files = self.pfind(
            basedir='basedir',
            filters=[{'filename': 'file3'}])
        self.assertEqual(files, ['dir/subdir/file3'])

    def test_with_basedir_and_join(self):
        files = self.pfind(
            join=True, basedir='basedir',
            filters=[{'filename': 'file3'}])
        self.assertEqual(files, ['basedir/dir/subdir/file3'])

    def test_with_maxdepth_is_1(self):
        files = self.pfind(
            filters=[{'filename': 'file1'}, {'maxdepth': 1}])
        self.assertEqual(files, ['file1'])

    def test_with_maxdepth_is_2(self):
        files = self.pfind(
            filters=[{'filename': 'file1'}, {'maxdepth': 2}])
        self.assertEqual(files, ['file1', 'dir/file1'])

    def test_with_mapper(self):
        mapper = lambda emitter: emitter.value(emitter.value() + '!')
        files = self.pfind(
            filters=[{'filename': 'file1'}],
            mappers=[mapper])
        self.assertEqual(files, ['file1!', 'dir/file1!'])

    def test_with_reducer(self):
        reducer = lambda files: list(files)[0]
        files = self.pfind(
            filters=[{'filename': 'file1'}, {'maxdepth': 1}],
            reducers=[reducer])
        self.assertEqual(files, 'file1')

    def test_with_reducer_and_fallback(self):
        reducer = lambda values: 1 / 0
        files = self.pfind(
            reducers=[reducer], fallback='fallback')
        self.assertEqual(files, 'fallback')
