import re
import unittest
from functools import partial
from box.functools import cachedproperty
from unittest.mock import Mock, patch, call
from importlib import import_module
component = import_module('run_find.strings.find')


class find_strings_Test(unittest.TestCase):

    # Actions

    def setUp(self):
        self.addCleanup(patch.stopall)
        self.filepathes = ['file1', 'file2']
        patch.object(component, 'open', self.mock_open).start()
        patch.object(component, 'find_files', self.mock_find_files).start()
        self.pfind = partial(component.find_strings, reducers=[list])

    # Helpers

    @cachedproperty
    def mock_open(self):
        # Function mock_open has different behaviour in Python 3.3/3.4:
        # In 3.3 position in "file" resets after every read()
        # In 3.4 position in "file" doesn't reset even after new open()
        # _open = mock_open(read_data='data')
        return Mock(
            return_value=Mock(
                __exit__=Mock(),
                __enter__=Mock(return_value=Mock(
                read=Mock(return_value='data')))))

    @cachedproperty
    def mock_find_files(self):
        return Mock(
            return_value=self.filepathes)

    # Tests

    def test(self):
        strings = self.pfind(basedir='basedir')
        self.assertEqual(strings, ['data', 'data'])
        self.mock_open.assert_has_calls(
            [call('basedir/file1'),
             call('basedir/file2')])

    def test_with_string(self):
        strings = self.pfind(string='data')
        self.assertEqual(strings, ['data', 'data'])

    def test_with_string_is_regex(self):
        strings = self.pfind(string=re.compile('data'))
        self.assertEqual(strings, ['data', 'data'])

    def test_with_string_is_regex_with_groups(self):
        strings = self.pfind(string=re.compile('(d|t)(a)'))
        self.assertEqual(strings, ['d', 'a', 't', 'a', 'd', 'a', 't', 'a'])

    def test_with_mapper(self):
        mapper = (lambda emitter:
            emitter.value(emitter.filepath + ':' + emitter.value()))
        strings = self.pfind(string='data', mappers=[mapper])
        self.assertEqual(strings, ['file1:data', 'file2:data'])

    def test_with_reducer_and_fallback(self):
        reducer = lambda values: 1 / 0
        strings = self.pfind(reducers=[reducer], fallback='fallback')
        self.assertEqual(strings, 'fallback')
