import unittest
from importlib import import_module
component = import_module('run_find.find')


class find_Test(unittest.TestCase):

    # Action

    def setUp(self):
        self.values = ['value1', 'value2']
        self.find = self.make_mock_find()

    # Helpers

    def make_mock_find(self):
        class mock_find(component.find):
            # Public
            def __init__(self, values, *args, **kwargs):
                self.__values = values
                super().__init__(*args, **kwargs)
            # Protected
            @property
            def _values(self):
                return self.__values
        return mock_find

    # Tests

    def test(self):
        result = list(self.find(self.values))
        self.assertEqual(result, ['value1', 'value2'])

    def test_with_mapper_using_set_and_get_value(self):
        mapper = lambda emitter: emitter.value(emitter.value() + '!')
        result = list(self.find(self.values, mappers=[mapper]))
        self.assertEqual(result, ['value1!', 'value2!'])

    def test_with_mapper_using_emit_with_condition(self):
        mapper1 = lambda emitter: emitter.emit('emitted1')
        mapper2 = (lambda emitter:
            emitter.emit('emitted2', emitter.value() == 'value1'))
        result = list(self.find(self.values, mappers=[mapper1, mapper2]))
        self.assertEqual(result, ['emitted1', 'emitted2', 'emitted1'])

    def test_with_mapper_using_skip(self):
        mapper = lambda emitter: emitter.skip()
        result = list(self.find(self.values, mappers=[mapper]))
        self.assertEqual(result, [])

    def test_with_mapper_using_stop(self):
        mapper = lambda emitter: emitter.stop()
        result = list(self.find(self.values, mappers=[mapper]))
        self.assertEqual(result, ['value1'])

    def test_with_mapper_using_stop_and_skip(self):
        # Test ensures that map will stop on step with stop and skip
        mapper1 = lambda emitter: emitter.stop()
        def mapper2(emitter):
            if emitter.value() == 'value1':
                emitter.skip()
        result = list(self.find(self.values, mappers=[mapper1, mapper2]))
        self.assertEqual(result, [])

    def test_with_reducer(self):
        reducer = lambda result: 'reduced'
        result = self.find(self.values, reducers=[reducer])
        self.assertEqual(result, 'reduced')

    def test_with_getfirst(self):
        result = self.find(self.values, getfirst=True)
        self.assertEqual(result, 'value1')

    def test_with_getfirst_and_not_values(self):
        self.assertRaises(Exception, self.find, [], getfirst=True)

    def test_with_fallback_is_none_and_error(self):
        self.assertRaises(Exception, self.find, [], getfirst=True)

    def test_with_fallback_is_value_and_error(self):
        result = self.find([], getfirst=True, fallback='fallback')
        self.assertEqual(result, 'fallback')

    def test_with_fallback_is_exception_and_error(self):
        self.assertRaises(RuntimeError,
            self.find, [], getfirst=True, fallback=RuntimeError())

    def test_with_fallback_is_callable_and_error(self):
        result = self.find([], getfirst=True, fallback=lambda excp: 'excp')
        self.assertEqual(result, 'excp')
